@@ -1,6 +1,7 @@
 #include "latexpackage.h"
 #include "latexcompleter_config.h"
 #include "latexparser/latexparser.h"
+#include "tablemanipulation.h"
 
 
 CommandDescription extractCommandDef(QString line, QString definition);
@@ -49,6 +50,7 @@ QStringList LatexPackage::keyToOptions(const QString &key)
 			elem = elem.left(i);
 		}
 		elem = elem.simplified();
+        elem = elem.remove(QChar(' '));
 		result[k] = elem;
 	}
 	return result;
@@ -63,10 +65,8 @@ void LatexPackage::unite(LatexPackage &add, bool forCompletion)
 	optionCommands.unite(add.optionCommands);
 	environmentAliases.unite(add.environmentAliases);
 #if (QT_VERSION<QT_VERSION_CHECK(5,15,0))
-    specialTreatmentCommands.unite(add.specialTreatmentCommands);
     specialDefCommands.unite(add.specialDefCommands);
 #else
-    specialTreatmentCommands.insert(add.specialTreatmentCommands);
     specialDefCommands.insert(add.specialDefCommands);
 #endif
 	commandDescriptions.unite(add.commandDescriptions); // overloaded unit, which does not overwrite well defined CDs with poorly defined ones
@@ -91,6 +91,9 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 	LatexPackage package;
 
 	QFile tagsfile("cwl:" + fileName);
+    if(QFileInfo(fileName).isAbsolute() && !tagsfile.exists()){
+        tagsfile.setFileName(fileName);
+    }
 	bool skipSection = false;
 	if (tagsfile.exists() && tagsfile.open(QFile::ReadOnly)) {
 		QString line;
@@ -98,14 +101,12 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 #if QT_VERSION < QT_VERSION_CHECK(6,0,0)
         stream.setCodec("UTF-8");
 #endif
-		QRegExp rxCom("^(\\\\\\w+\\*?)(\\[.+\\])*\\{(.*)\\}");  // expression for \cmd[opt]{arg} (cmd may be starred, [opt] can appear arbitrary often)
-		QRegExp rxCom2("^(\\\\\\w+\\*?)\\[(.+)\\]");            // expression for \cmd[opt]      (cmd may be starred)
-		QRegExp rxCom3("^(\\\\\\w+\\*?)");                      // expression for \cmd           (cmd may be starred)
-		rxCom.setMinimal(true);
+        static const QRegularExpression rxCom("^(\\\\\\w+\\*?)(\\[.+?\\])*?\\{(.*?)\\}");  // expression for \cmd[opt]{arg} (cmd may be starred, [opt] can appear arbitrary often)
+        static const QRegularExpression rxCom2("^(\\\\\\w+\\*?)\\[(.+)\\]");            // expression for \cmd[opt]      (cmd may be starred)
+        static const QRegularExpression rxCom3("^(\\\\\\w+\\*?)");                      // expression for \cmd           (cmd may be starred)
 		QStringList keywords;
 		keywords << "text" << "title" << "%<text%>" << "%<title%>";
-		QStringList specialTreatment;
-		specialTreatment << "color";
+
 		QString keyvals;
         package.containsOptionalSections=false;
 		while (!stream.atEnd()) {
@@ -148,11 +149,14 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 				QStringList l_cmds=keyvals.split(',');
 				QString key;
 				CommandDescription cd = extractCommandDefKeyVal(line, key);
-				for(const QString &elem:l_cmds){
+                for(QString elem:l_cmds){
 					package.possibleCommands["key%" + elem] << line;
-					if (cd.args > 0) {
+                    if (cd.arguments.size() > 0) {
 						if (key.endsWith("="))
 							key.chop(1);
+                        if(elem.endsWith("#c")){
+                            elem.chop(2);
+                        }
 						package.commandDescriptions.insert(elem + "/" + key, cd);
 					}
 				}
@@ -204,44 +208,48 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 				}
 
 				// parse for spell checkable commands
-				int res = rxCom.indexIn(line);
-				if (keywords.contains(rxCom.cap(3))) {
-					package.optionCommands << rxCom.cap(1);
+                QRegularExpressionMatch rxComMatch = rxCom.match(line);
+                int res = rxComMatch.capturedStart();
+                if (keywords.contains(rxComMatch.captured(3))) {
+                    package.optionCommands << rxComMatch.captured(1);
 				}
-
-				if (specialTreatment.contains(rxCom.cap(3))) {
-					package.specialTreatmentCommands[rxCom.cap(1)].insert(qMakePair(rxCom.cap(3), 1));
-				}
-				rxCom2.indexIn(line); // for commands which don't have a braces part e.g. \item[text]
-				int res3 = rxCom3.indexIn(line); // for commands which don't have a options either e.g. \node (asas)
+                QRegularExpressionMatch rxComMatch2 = rxCom2.match(line); // for commands which don't have a braces part e.g. \item[text]
+                QRegularExpressionMatch rxComMatch3 = rxCom3.match(line); // for commands which don't have a options either e.g. \node (asas)
+                int res3 = rxComMatch3.capturedStart();
 
 				// get commandDefinition
 				CommandDescription cd = extractCommandDef(line, valid);
+                if(valid.startsWith("beginEnv")){
+                    package.possibleCommands["%beginEnv"]<<line;
+                    package.environmentAliases.insert(line, definition);
+                    valid=valid.mid(8); // maintain additional classifiers
+                }
+                if(valid.startsWith("endEnv")){
+                    package.possibleCommands["%endEnv"]<<line;
+                    package.environmentAliases.insert(line, definition);
+                    valid=valid.mid(6); // maintain additional classifiers
+                }
 				if(valid.contains('K')){
 					// bracket command like \left etc
 					cd.bracketCommand=true;
 					valid.remove("K");
 				}
-				QString cmd = rxCom3.cap(1);
+                QString cmd = rxComMatch3.captured(1);
 				if (cmd == "\\begin") {
 					if (!package.commandDescriptions.contains(cmd)) {
 						// one insertion of a general \begin-command
 						CommandDescription cd;
-						cd.args = 1;
-						cd.argTypes << Token::beginEnv;
+                        cd.arguments << ArgumentDescription{ArgumentDescription::MANDATORY, Token::beginEnv};
 						package.commandDescriptions.insert(cmd, cd);
 					}
-					cmd = rxCom.cap();
+                    cmd = rxComMatch.captured();
 				}
 				if (package.commandDescriptions.contains(cmd)) {
 					CommandDescription cd_old = package.commandDescriptions.value(cmd);
-					if (cd_old.args == cd.args && cd_old.optionalArgs > cd.optionalArgs ) {
+                    if (cd_old.arguments.size()  > cd.arguments.size() ) {
 						cd = cd_old;
 					}
-					if (cd_old.args == cd.args && cd_old.optionalArgs == cd.optionalArgs && cd_old.overlayArgs > cd.overlayArgs ) {
-						cd = cd_old;
-					}
-					if (cd_old.args < cd.args && cd_old.args > 0) {
+                    if (cd_old.args() < cd.args() && cd_old.args() > 0) {
 						cd = cd_old;
 #ifndef QT_NO_DEBUG
 						qDebug() << "inconsistent command arguments:" << cmd << fileName;
@@ -251,9 +259,9 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 #endif
 					}
 
-					if (cd_old.args > cd.args) {
+                    if (cd_old.args() > cd.args()) {
 #ifndef QT_NO_DEBUG
-						if (cd.args > 0) {
+                        if (cd.args() > 0) {
 							qDebug() << "inconsistent command arguments:" << cmd << fileName;
 							// commands with different numbers of mandatory arguments are not distinguished by the parser and lead to unreliable results.
 							// the lower numer of mandatory arguments is handled only (however not an command with zero arguments)
@@ -262,8 +270,8 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 #endif
 						cd = cd_old;
 					}
-
 				}
+
 				if(!valid.contains('M')){
 					package.commandDescriptions.insert(cmd, cd);
 				}else{
@@ -271,41 +279,56 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 					valid.remove('M');
 				}
 
-
 				valid.remove('N'); // remove newtheorem declaration
 
-
-				if (keywords.contains(rxCom2.cap(2))) {
-					package.optionCommands << rxCom2.cap(1);
+                if (keywords.contains(rxComMatch2.captured(2))) {
+                    package.optionCommands << rxComMatch2.captured(1);
 				}
+                if (valid.contains("d1")) { // definition command for newtcbox and similar. It automatically will add an extra mandatory argument for the completer
+                    if (res > -1) {
+                        package.possibleCommands["%definition"] << rxComMatch.captured(1);
+                        package.possibleCommands["%definition1"] << rxComMatch.captured(1);
+                    }
+                    valid.remove("d1");
+                }
 				if (valid.contains('d')) { // definition command
 					if (res > -1) {
-						package.possibleCommands["%definition"] << rxCom.cap(1);
+                        package.possibleCommands["%definition"] << rxComMatch.captured(1);
 					}
 					valid.remove('d');
 				}
 				if (valid.contains('i')) { // include like command
 					if (res > -1) {
-						package.possibleCommands["%include"] << rxCom.cap(1);
+                        package.possibleCommands["%include"] << rxComMatch.captured(1);
 					}
 					valid.remove('i');
 				}
+                if (valid.contains('I')) { // include like command
+                    if (res > -1) {
+                        package.possibleCommands["%import"] << rxComMatch.captured(1);
+                    }
+                    valid.remove('I');
+                }
 				if (valid.contains('l')) { // label command
 					if (res > -1) {
-						package.possibleCommands["%label"] << rxCom.cap(1);
+                        package.possibleCommands["%label"] << rxComMatch.captured(1);
 					}
 					valid.remove('l');
 				}
 				if (valid.contains('r')) { // ref command
 					if (res > -1) {
-						package.possibleCommands["%ref"] << rxCom.cap(1);
+                        package.possibleCommands["%ref"] << rxComMatch.captured(1);
 
                         QRegularExpression re{"{.*?}"};
                         QRegularExpressionMatchIterator it = re.globalMatch(line);
                         QRegularExpressionMatch match;
-                        for(int i=0;i<cd.argTypes.size();++i){
+                        for(int i=0;i<cd.arguments.size();++i){
+                            if(!it.hasNext()) {
+                                match=QRegularExpressionMatch();
+                                break;
+                            }
                             match = it.next();
-                            if(cd.argTypes[i]==Token::labelRef)
+                            if(cd.arguments[i].tokenType==Token::labelRef)
                                 break;
                         }
                         if(match.hasMatch()){
@@ -369,17 +392,17 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 				}
 				if (valid.contains('V')) { // verbatim command
 					if (res > -1) {
-						package.possibleCommands["%verbatimEnv"] << rxCom.cap(3);
+                        package.possibleCommands["%verbatimEnv"] << rxComMatch.captured(3);
 						env << "verbatim";
 					}
 					valid.remove('V');
 				}
 				if (valid.contains('s')) { // special def
 					if (res > -1) {
-						package.specialDefCommands.insert(rxCom.cap(1), definition);
+                        package.specialDefCommands.insert(rxComMatch.captured(1), definition);
 					} else {
 						if (res3 > -1)
-							package.specialDefCommands.insert(rxCom3.cap(1), definition);
+                            package.specialDefCommands.insert(rxComMatch3.captured(1), definition);
 					}
 					if (definition.startsWith('%')) {
 						if (config)
@@ -405,46 +428,52 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
                         QRegularExpression re{"{.*?}"};
                         QRegularExpressionMatchIterator it = re.globalMatch(line);
                         QRegularExpressionMatch match;
-                        for(int i=0;i<cd.argTypes.size();++i){
+                        for(int i=0;i<cd.arguments.size();++i){
+                            if(!it.hasNext()) {
+                                match=QRegularExpressionMatch();
+                                break;
+                            }
                             match = it.next();
-                            if(cd.argTypes[i]==Token::bibItem)
+                            if(cd.arguments[i].tokenType==Token::bibItem)
                                 break;
                         }
-                        package.possibleCommands["%cite"] << rxCom.cap(1);
+                        package.possibleCommands["%cite"] << rxComMatch.captured(1);
                         if (!line.startsWith("\\begin")) // HANDLE begin extra
-                            package.possibleCommands["%citeExtendedCommand"] << rxCom.cap(1);
-                        line.replace(match.capturedStart(),match.capturedLength(),"{@}");
+                            package.possibleCommands["%citeExtendedCommand"] << rxComMatch.captured(1);
+                        if(match.hasMatch()){
+                            line.replace(match.capturedStart(),match.capturedLength(),"{@}");
+                        }
 					}
 					valid.remove('C');
 				}
 				if (valid.contains('g')) { // definition command
 					if (res > -1) {
-						package.possibleCommands["%graphics"] << rxCom.cap(1);
+                        package.possibleCommands["%graphics"] << rxComMatch.captured(1);
 					}
 					valid.remove('g');
 				}
 				if (valid.contains('u')) { // usepackage command
 					if (res > -1) {
-						package.possibleCommands["%usepackage"] << rxCom.cap(1);
+                        package.possibleCommands["%usepackage"] << rxComMatch.captured(1);
 					}
 					valid.remove('u');
 				}
 				if (valid.contains('b')) { // usepackage command
 					if (res > -1) {
-						package.possibleCommands["%bibliography"] << rxCom.cap(1);
-						package.possibleCommands["%file"] << rxCom.cap(1);
+                        package.possibleCommands["%bibliography"] << rxComMatch.captured(1);
+                        package.possibleCommands["%file"] << rxComMatch.captured(1);
 					}
 					valid.remove('b');
 				}
 				if (valid.contains('U')) { // url command
 					if (res > -1) {
-						package.possibleCommands["%url"] << rxCom.cap(1);
+                        package.possibleCommands["%url"] << rxComMatch.captured(1);
 					}
 					valid.remove('U');
 				}
 				if (valid.contains('D')) { // todo command
 					if (res > -1) {
-						package.possibleCommands["%todo"] << rxCom.cap(1);
+                        package.possibleCommands["%todo"] << rxComMatch.captured(1);
 					}
 					valid.remove('D');
 				}
@@ -460,10 +489,10 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 				// will be extended to distinguish between normal and math commands
 				if (valid.isEmpty() || valid.contains('n')) {
 					if (res > -1) {
-						if (rxCom.cap(1) == "\\begin" || rxCom.cap(1) == "\\end") {
-							package.possibleCommands["normal"] << rxCom.cap(1) + "{" + rxCom.cap(3) + "}";
+                        if (rxComMatch.captured(1) == "\\begin" || rxComMatch.captured(1) == "\\end") {
+                            package.possibleCommands["normal"] << rxComMatch.captured(1) + "{" + rxComMatch.captured(3) + "}";
 						} else {
-							package.possibleCommands["normal"] << rxCom.cap(1);
+                            package.possibleCommands["normal"] << rxComMatch.captured(1);
 						}
 					} else {
 						if (!cmd.isEmpty())
@@ -474,10 +503,10 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 				}
 				if (valid.contains('m')) { // math commands
 					if (res > -1) {
-						if (rxCom.cap(1) == "\\begin" || rxCom.cap(1) == "\\end") {
-							package.possibleCommands["math"] << rxCom.cap(1) + "{" + rxCom.cap(3) + "}";
+                        if (rxComMatch.captured(1) == "\\begin" || rxComMatch.captured(1) == "\\end") {
+                            package.possibleCommands["math"] << rxComMatch.captured(1) + "{" + rxComMatch.captured(3) + "}";
 						} else {
-							package.possibleCommands["math"] << rxCom.cap(1);
+                            package.possibleCommands["math"] << rxComMatch.captured(1);
 						}
 					} else {
 						if (!cmd.isEmpty())
@@ -488,12 +517,12 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 				}
 				if (valid.contains('t')) { // tabular commands
 					if (res > -1) {
-						if (rxCom.cap(1) == "\\begin" || rxCom.cap(1) == "\\end") {
-							package.possibleCommands["tabular"] << rxCom.cap(1) + "{" + rxCom.cap(3) + "}";
-							package.possibleCommands["array"] << rxCom.cap(1) + "{" + rxCom.cap(3) + "}";
+                        if (rxComMatch.captured(1) == "\\begin" || rxComMatch.captured(1) == "\\end") {
+                            package.possibleCommands["tabular"] << rxComMatch.captured(1) + "{" + rxComMatch.captured(3) + "}";
+                            package.possibleCommands["array"] << rxComMatch.captured(1) + "{" + rxComMatch.captured(3) + "}";
 						} else {
-							package.possibleCommands["tabular"] << rxCom.cap(1);
-							package.possibleCommands["array"] << rxCom.cap(1);
+                            package.possibleCommands["tabular"] << rxComMatch.captured(1);
+                            package.possibleCommands["array"] << rxComMatch.captured(1);
 						}
 					} else {
 						if (cmd.isEmpty())
@@ -507,13 +536,13 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 						package.possibleCommands["tabbing"] << cmd;
 					}
 				}
-				if (valid.contains('e') && !env.isEmpty()) { // tabbing support
+                if (valid.contains('e') && !env.isEmpty()) { // restrict to environments
 					if (res == -1) {
 						foreach (const QString &elem, env)
 							package.possibleCommands[elem] << cmd;
 					} else {
-						QString cmd = rxCom.cap(1);
-						QString envName = rxCom.cap(3);
+                        QString cmd = rxComMatch.captured(1);
+                        QString envName = rxComMatch.captured(3);
 						if (cmd == "\\begin" || cmd == "\\end") {
 							cmd += "{" + envName + "}";
 						}
@@ -523,11 +552,18 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 				}
 				if (!valid.contains('e') && !env.isEmpty()) { // set env alias
 					if (res > -1) {
-						if (rxCom.cap(1) == "\\begin") {
-							QString envName = rxCom.cap(3);
+                        if (rxComMatch.captured(1) == "\\begin") {
+                            QString envName = rxComMatch.captured(3);
 							if (!envName.isEmpty()) {
-								foreach (const QString &elem, env)
-									package.environmentAliases.insert(rxCom.cap(3), elem);
+                                foreach (const QString &elem, env){
+                                    package.environmentAliases.insert(rxComMatch.captured(3), elem);
+                                    if(elem=="tabular"){
+                                        LatexTables::tabularNames.insert(envName);
+                                    }
+                                    if(elem=="array"){
+                                        LatexTables::mathTables.insert(envName);
+                                    }
+                                }
 							}
 						}
 					}
@@ -588,6 +624,9 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 					it->snippetLength = len;
 					it->usageCount = uncommon ? -1 : 0;
 					it->type = type;
+                    if(valid.contains('e') && !env.isEmpty()){
+                        it->environmentRestriction = env.first(); // only use first env for now
+                    }
 					if (config) {
 						QList<QPair<int, int> >res = config->usage.values(hash);
 						foreach (const PairIntInt &elem, res) {
@@ -612,7 +651,7 @@ LatexPackage loadCwlFile(const QString fileName, LatexCompleterConfig *config, Q
 }
 
 
-Token::TokenType tokenTypeFromCwlArg(QString arg, QString definition)
+Token::TokenType tokenTypeFromCwlArg(QString arg, QString &definition)
 {
 	int i = arg.indexOf('%');
 	// type from suffix
@@ -623,6 +662,7 @@ Token::TokenType tokenTypeFromCwlArg(QString arg, QString definition)
 		if (suffix == "%title") return Token::title;
 		if (suffix == "%short title") return Token::shorttitle;
 		if (suffix == "%todo") return Token::todo;
+        if (suffix == "%file") return Token::file;
 		if (suffix == "%l") return Token::width;
 		if (suffix == "%cmd") return Token::def;
         if (suffix == "%definition") return Token::definition;
@@ -631,6 +671,9 @@ Token::TokenType tokenTypeFromCwlArg(QString arg, QString definition)
 		if ((suffix == "%envname") && definition.contains('N')) return Token::newTheorem;
 		if (suffix == "%ref") return Token::labelRef;
 		if (suffix == "%labeldef") return Token::label;
+        if (suffix == "%specialDef"){
+            return Token::defSpecialArg;
+        }
 		if (suffix == "%special") {
 			Token::TokenType type = Token::specialArg;
 			arg.chop(8);
@@ -679,6 +722,7 @@ Token::TokenType tokenTypeFromCwlArg(QString arg, QString definition)
 	if (arg == "labellist") return Token::labelRefList;
     if (arg == "verbatimSymbol") return Token::verbatimStart;
 	if (arg.contains("overlay specification")) return Token::overlay;
+
 	return Token::generalArg;
 }
 
@@ -725,9 +769,9 @@ argument name | description
  */
 CommandDescription extractCommandDef(QString line, QString definition)
 {
-	QRegExp rxCom("^(\\\\\\w+\\*?)");
-	int i = rxCom.indexIn(line);
-	QString command = rxCom.cap();
+    QRegularExpression rxCom("^(\\\\\\w+\\*?)");
+    QRegularExpressionMatch rxComMatch = rxCom.match(line);
+    QString command = rxComMatch.captured();
 	line = line.mid(command.length());
 	const QString specialChars = "{[(<";
 	const QString specialChars2 = "}])>";
@@ -739,8 +783,16 @@ CommandDescription extractCommandDef(QString line, QString definition)
 	while (specialChars.contains(c)) {
 		int j = specialChars.indexOf(c);
 		QChar closingChar = specialChars2.at(j);
-		i = line.indexOf(closingChar);
-		QString arg = line.mid(1, i - 1);
+        int i = line.indexOf(closingChar);
+        QString arg = line.mid(1, i - 1);
+        // special case [< .. >] for default overlay specification
+        if(j==1 && i>3){
+            if(line.at(1)=='<' && line.at(i-1)=='>'){
+                j=4;
+                arg = line.mid(2, i - 2);
+            }
+        }
+
 		Token::TokenType type = Token::generalArg; // assume that unknown argument is not a text
 		if (loop == 1 && command == "\\begin") {
 			type = Token::beginEnv;
@@ -752,21 +804,20 @@ CommandDescription extractCommandDef(QString line, QString definition)
 		if (!arg.isEmpty()) { //ignore empty arguments
 			switch (j) {
 			case 0:
-				cd.args = cd.args + 1;
-				cd.argTypes.append(type);
+                cd.arguments<<ArgumentDescription{ArgumentDescription::MANDATORY, type};
 				break;
 			case 1:
-				cd.optionalArgs = cd.optionalArgs + 1;
-				cd.optTypes.append(type);
+                cd.arguments<<ArgumentDescription{ArgumentDescription::OPTIONAL, type};
 				break;
 			case 2:
-				cd.bracketArgs = cd.bracketArgs + 1;
-				cd.bracketTypes.append(type);
-				break;
+                cd.arguments<<ArgumentDescription{ArgumentDescription::BRACKET, type};
+                break;
 			case 3:
-				cd.overlayArgs = cd.overlayArgs + 1;
-				cd.overlayTypes.append(type);
-				break;
+                cd.arguments<<ArgumentDescription{ArgumentDescription::OVERLAY, type};
+                break;
+            case 4:
+                cd.arguments<<ArgumentDescription{ArgumentDescription::DEFAULT_OVERLAY, type};
+                break;
 			default:
 				break;
 			}
@@ -789,14 +840,20 @@ CommandDescription extractCommandDefKeyVal(QString line, QString &key)
 	if (i < 0)
 		return cd;
 	key = line.left(i);
+    int j= key.indexOf("=");
+    if(i>=0){
+        // also cut possible "=" and subsequent values
+        key = key.left(j);
+    }
 	QString vals = line.mid(i + 1);
 	if (vals == "#L") {
-		cd.args = 1;
-		cd.argTypes << Token::width;
+        cd.arguments={ArgumentDescription{ArgumentDescription::MANDATORY, Token::width}};
 	}
 	if (vals == "#l") {
-		cd.args = 1;
-		cd.argTypes << Token::label;
+        cd.arguments={ArgumentDescription{ArgumentDescription::MANDATORY, Token::label}};
 	}
+    if (vals == "#d") {
+        cd.arguments={ArgumentDescription{ArgumentDescription::MANDATORY, Token::definition}};
+    }
 	return cd;
 }
