@@ -11,13 +11,10 @@
 namespace Parsing {
 
 
-//const int RUNAWAYLIMIT = 30; // limit lines to process multi-line arguments in order to prevent processing to the end of document if the argument is unclosed
-
-
 /*!
  * Realizes the first pass lexing
  * Following functionality is implemented:
- * + separate the the text into words,symbols
+ * + separate the text into words,symbols
  * + assign each symbol/word a basic context like word,command,symbol,open/close brace etc.
  * + set tokenlist as LEXER_RAW_COOKIE on line
  * + remove cookie LEXER_COOKIE (as it is invalid)
@@ -87,6 +84,13 @@ TokenList simpleLexLatexLine(QDocumentLineHandle *dlh)
 					continue;
 				}
 				if (c.isLetter() && present.type == Token::number) {
+                    // special case 0x... 0b....
+                    if((c=='x' || c=='b')   && i-present.start==1 && s[present.start]=='0'){
+                        continue;
+                    }
+                    if(QString("abcdef").contains(c.toLower()) && s[present.start]=='0' && s[present.start+1]=='x'){
+                        continue;
+                    }
 					present.length = i - present.start;
 					lexed.append(present);
 					present.start = i;
@@ -146,7 +150,7 @@ TokenList simpleLexLatexLine(QDocumentLineHandle *dlh)
 }
 
 
-bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, CommandStack &commandStack, const LatexParser &lp)
+bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, CommandStack &commandStack, const QSharedPointer<LatexParser> lp)
 {
 	if (!dlh)
 	    return false;
@@ -171,6 +175,17 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
 	int lastEqual = -1e6;
     int commentStart=-1;
 	QString keyName;
+    // extra stack for storing lastComma/lastEqual in case of new argument is introduced
+    // this works single line only (for now)
+    // see issue #4074
+    struct ArgumentSeparator
+    {
+        int lastComma;
+        int lastEqual;
+    };
+    QStack<ArgumentSeparator> argumentStack;
+
+    bool unknownCommandsPresent = false;
 
 	int lineLength=line.length();
 
@@ -206,6 +221,7 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                     tk1.start=tk.start;
                     tk1.length=1;
                     tk1.type=Token::verbatim;
+                    tk1.dlh=tk.dlh;
                     tk.type=Token::symbol;
                     tk.subtype=Token::verbatimStop;
                     tk.level = level;
@@ -237,11 +253,27 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                 continue;
             Token tk2 = tl.at(i + 1);
             Token tk3 = tl.at(i + 2);
-            if (tk2.type == Token::openBrace && tk3.type == Token::word) {
-                QString env = line.mid(tk3.start, tk3.length);
-                if (lp.possibleCommands["%verbatimEnv"].contains(env)) { // incomplete check if closing corresponds to open !
-                    verbatimMode = false;
-                    stack.pop();
+            // search for closing brace (#2528)
+            Token tk_closing;
+            bool closingFound=false;
+            for(int j=i+3;j<tl.length();++j){
+                tk_closing=tl.at(j);
+                if(tk_closing.type == Token::closeBrace){
+                    closingFound=true;
+                    break;
+                }
+            }
+            if (tk2.type == Token::openBrace && closingFound) {
+                QString env = line.mid(tk3.start, tk_closing.start-tk3.start);
+                if (lp->possibleCommands["%verbatimEnv"].contains(env)) { // incomplete check if closing corresponds to open !
+                    Token verbatimStart=stack.top();
+                    // second option, env in optionalCommandName
+                    if(verbatimStart.optionalCommandName.isEmpty() || verbatimStart.optionalCommandName==env){
+                        verbatimMode = false;
+                        stack.pop();
+                    }else{
+                        continue;
+                    }
                 } else
                     continue;
             } else
@@ -314,9 +346,10 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                 tk.subtype = stack.top().subtype;
                 if (tk.subtype == Token::keyValArg && lastEqual > -1) {
                     tk.subtype = Token::keyVal_val;
-                    if (!commandStack.isEmpty() && lp.commandDefs.contains(commandStack.top().optionalCommandName + "/" + keyName)) {
-                        CommandDescription cd = lp.commandDefs.value(commandStack.top().optionalCommandName + "/" + keyName);
-                        tk.subtype = cd.argTypes.value(0, Token::keyVal_val);
+                    if (!commandStack.isEmpty() && lp->commandDefs.contains(commandStack.top().optionalCommandName + "/" + keyName)) {
+                        CommandDescription cd = lp->commandDefs.value(commandStack.top().optionalCommandName + "/" + keyName);
+                        ArgumentDescription ad=cd.arguments.value(0, ArgumentDescription{ArgumentDescription::MANDATORY,Token::keyVal_val});
+                        tk.subtype = ad.tokenType;
                     }
                 }
 
@@ -325,20 +358,26 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
             if (!commandStack.isEmpty() && commandStack.top().level == level) {
                 //possible command argument without brackets
                 CommandDescription &cd = commandStack.top();
-                if (cd.args > 0) {
-                    cd.optionalArgs = 0; // no optional arguments after mandatory
-                    cd.bracketArgs = 0;
-                    cd.args--;
-                    tk.subtype = cd.argTypes.takeFirst();
-                    tk.level++;
+                // skip over optional arguments
+                while(!cd.arguments.isEmpty() && (cd.arguments.first().type >= ArgumentDescription::OPTIONAL)) {
+                    cd.arguments.takeFirst();
                 }
-                if (cd.args <= 0) {
+                // check for mandatory argument
+                if (!cd.arguments.isEmpty() && cd.arguments[0].type==ArgumentDescription::MANDATORY) {
+                    ArgumentDescription ad=cd.arguments.takeFirst();
+                    tk.subtype = ad.tokenType;
+                    tk.level++;
+                }else{
                     // unknown arg, stop handling this command
+                    cd.arguments.clear();
+                }
+                if (cd.arguments.isEmpty()) {
+                    // command has been handled
                     commandStack.pop();
                 }
             }
-            if (lp.commandDefs.contains(command) && tk.subtype != Token::definition) {
-                CommandDescription cd = lp.commandDefs.value(command);
+            if (lp->commandDefs.contains(command) && tk.subtype != Token::definition) {
+                CommandDescription cd = lp->commandDefs.value(command);
                 cd.level = level;
                 if(cd.bracketCommand){
                     //command like \left
@@ -354,13 +393,17 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                     }
 
                 }
-                if ((cd.args > 0 || cd.optionalArgs > 0 || cd.bracketArgs > 0 || cd.overlayArgs > 0) && tk.subtype != Token::def) { // don't interpret commands in definition (\newcommand{def})
+                if (cd.arguments.size() > 0 && tk.subtype != Token::def) { // don't interpret commands in definition (\newcommand{def})
                     cd.optionalCommandName=command;
                     commandStack.push(cd);
                 }
             } else {
-                if(tk.type==Token::command){
-                    tk.type = Token::commandUnknown;
+                if(tk.type==Token::command && tk.subtype == Token::none){
+                    if(command!="\\\\"){
+                        // skip "\\"
+                        tk.type = Token::commandUnknown;
+                        unknownCommandsPresent=true;
+                    }
                 }
             }
             lexed << tk;
@@ -370,22 +413,64 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
             if (!commandStack.isEmpty() && commandStack.top().level == level) {
                 CommandDescription &cd = commandStack.top();
                 if (tk.type == Token::openBrace) {
-                    if (cd.args > 0) {
-                        //cd.optionalArgs=0; // argument order (option/mandatory) is not checked, e.g \newcommad{cmd}[argNumber][default]{definition}
-                        cd.args--;
-                        tk.subtype = cd.argTypes.takeFirst();
+                    // skip over optional arguments, here assume optional arguments as [],<> and ()
+                    while(!cd.arguments.isEmpty() && (cd.arguments.first().type >= ArgumentDescription::OPTIONAL) ) {
+                        cd.arguments.takeFirst();
+                    }
+                    // check for mandatory argument
+                    if (!cd.arguments.isEmpty() && cd.arguments.first().type == ArgumentDescription::MANDATORY) {
+                        ArgumentDescription ad= cd.arguments.takeFirst();
+                        tk.subtype = ad.tokenType;
                     } else {
-                        // ignore, but make sure that optional arguments are impossible after that (see #2046)
-                        commandStack.pop();
+                        // handle as independet braces, like commandless brace below
+                        // e.g. \hline {... (\hline accepts optional arguments, but no braces)
+                        tk.level = level;
+                        tk.argLevel = ConfigManager::RUNAWAYLIMIT; // run-away prevention, needs to be >0 as otherwise closing barces are misinterpreted
+                        if (!stack.isEmpty()) {
+                            tk.subtype = stack.top().subtype;
+                            if(tk.subtype==Token::text){
+                                tk.subtype=Token::none; //avoid assignening text subtype to just arbitrary braces inside an argument, see #3040 (#2603)
+                            }
+                            if(stack.top().subtype==Token::keyValArg){
+                                // still the generic argument, needs to be broken down to key or val
+                                if(lastComma>0){
+                                    // -> val
+                                    tk.subtype=Token::keyVal_val;
+                                    QString cmd=lexed[lastComma].optionalCommandName;
+                                    QString key=line.mid(lexed[lastComma].start, lexed[lastComma].length);
+                                    tk.optionalCommandName=cmd+"/"+key;
+                                }else{
+                                    tk.subtype=Token::keyVal_key; // not sure if that is a real scenario
+                                }
+                            }
+                        }
+                        stack.push(tk);
                         lexed << tk;
                         continue;
                     }
                 }
                 if (tk.type == Token::openSquare) {
-                    if (cd.optionalArgs > 0) {
-                        cd.optionalArgs--;
-                        tk.subtype = cd.optTypes.takeFirst();
-                    } else {
+                    bool handled=false;
+                    while(!handled && !cd.arguments.isEmpty() && (cd.arguments.first().type == ArgumentDescription::OVERLAY || cd.arguments.first().type == ArgumentDescription::BRACKET)) {
+                        cd.arguments.takeFirst();
+                    }
+                    if (!cd.arguments.isEmpty() && cd.arguments.first().type == ArgumentDescription::DEFAULT_OVERLAY) {
+                        // check next token to be less
+                        if (i + 1 < tl.length() && tl[i + 1].type == Token::less) {
+                            ArgumentDescription ad= cd.arguments.takeFirst();
+                            tk.subtype = ad.tokenType;
+                            ++i;
+                            handled=true;
+                        } else {
+                            cd.arguments.takeFirst();
+                        }
+                    }
+                    if (!handled && !cd.arguments.isEmpty() && cd.arguments.first().type == ArgumentDescription::OPTIONAL) {
+                        ArgumentDescription ad= cd.arguments.takeFirst();
+                        tk.subtype = ad.tokenType;
+                        handled=true;
+                    }
+                    if(!handled){
                         // unexpected optional argument
                         // ignore
                         lexed << tk;
@@ -393,18 +478,23 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                     }
                 }
                 if (tk.type == Token::openBracket) {
-                    if (cd.bracketArgs > 0) {
-                        cd.bracketArgs--;
-                        tk.subtype = cd.bracketTypes.takeFirst();
+                    // skip over optional arguments
+                    while(!cd.arguments.isEmpty() && (cd.arguments.first().type == ArgumentDescription::OPTIONAL || cd.arguments.first().type == ArgumentDescription::OVERLAY || cd.arguments.first().type == ArgumentDescription::DEFAULT_OVERLAY)) {
+                        cd.arguments.takeFirst();
+                    }
+                    // check for mandatory argument
+                    if (!cd.arguments.isEmpty() && cd.arguments.first().type == ArgumentDescription::BRACKET) {
+                        ArgumentDescription ad= cd.arguments.takeFirst();
+                        tk.subtype = ad.tokenType;
                     } else {
                         lexed << tk;
                         continue;
                     }
                 }
                 if (tk.type == Token::less) {
-                    if (cd.overlayArgs > 0) {
-                        cd.overlayArgs--;
-                        tk.subtype = cd.overlayTypes.takeFirst();
+                    if (!cd.arguments.isEmpty() && cd.arguments.first().type == ArgumentDescription::OVERLAY) {
+                        ArgumentDescription ad= cd.arguments.takeFirst();
+                        tk.subtype = ad.tokenType;
                     } else {
                         lexed << tk;
                         continue;
@@ -421,6 +511,13 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                 tk.level++;
                 lexed << tk;
                 level++;
+                // stack lastEqual/lastComma
+                ArgumentSeparator argSep;
+                argSep.lastComma=lastComma;
+                argSep.lastEqual=lastEqual;
+                argumentStack.push(argSep);
+                lastComma = -1;
+                lastEqual = -1e6;
             }else{
                 if(tk.type==Token::openBrace){ // check braces within arguments, not brackets/squareBrackets
                     //level++; // not an argument
@@ -428,7 +525,10 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                     tk.argLevel = ConfigManager::RUNAWAYLIMIT; // run-away prevention, needs to be >0 as otherwise closing barces are misinterpreted
                     if (!stack.isEmpty()) {
                         tk.subtype = stack.top().subtype;
-                        if(tk.subtype==Token::keyValArg){
+                        if(tk.subtype==Token::text){
+                            tk.subtype=Token::none; //avoid assignening text subtype to just arbitrary braces inside an argument, see #3040 (#2603)
+                        }
+                        if(stack.top().subtype==Token::keyValArg){
                             // still the generic argument, needs to be broken down to key or val
                             if(lastComma>0){
                                 // -> val
@@ -455,6 +555,20 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                 if (stack.top().type != Token::opposite(tk.type))
                     continue; //closing bracket/> is ignored if no correct open is present
             }
+            // special treatment for beamer default overlay [< >]
+            if(tk.type==Token::greater && stack.top().type == Token::openSquare){
+                if(!commandStack.isEmpty() && commandStack.top().arguments.size()>0 && commandStack.top().arguments.first().type==ArgumentDescription::DEFAULT_OVERLAY){
+                    // check if closing bracket is expected
+                    if(i+1<tl.length() && tl[i+1].type==Token::closeSquareBracket){
+                        // closing bracket found
+                        ++i;
+                        tk=tl[i]; // use outer closing square bracket
+                    }else{
+                        // closing bracket not found, ignore token
+                        continue;
+                    }
+                }
+            }
             if (!stack.isEmpty() && stack.top().type == Token::opposite(tk.type)) {
                 Token tk1 = stack.pop();
 
@@ -479,6 +593,13 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                     }
                     lastEqual = -1e6;
                 }
+                // handle stacked lastComma/lastEqual
+                if(!argumentStack.isEmpty()){
+                    ArgumentSeparator argSep=argumentStack.pop();
+                    lastComma=argSep.lastComma;
+                    lastEqual=argSep.lastEqual;
+                }
+
                 if (tk1.dlh == dlh) { // same line
                     int j = lexed.length() - 1;
                     while (j >= 0 && lexed.at(j).start > tk1.start)
@@ -501,14 +622,13 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                             if (tk2.type == Token::beginEnv) {
                                 // special treatment for \begin ...
                                 QString env = line.mid(tk2.start, tk2.length);
-                                CommandDescription cd = lp.commandDefs.value("\\begin{" + env + "}", CommandDescription());
+                                CommandDescription cd = lp->commandDefs.value("\\begin{" + env + "}", CommandDescription());
                                 // special treatment for verbatim
-                                if (lp.possibleCommands["%verbatimEnv"].contains(env)) {
-                                    if(cd.args==1 && cd.optionalArgs==1 && i<(tl.length()-1) && tl[i+1].type==Token::openSquare){ // next Token needs to be [ i.e. optional arg, otherwise start verbatim directly
+                                if (lp->possibleCommands["%verbatimEnv"].contains(env)) {
+                                    if(cd.args()==1 && cd.args(ArgumentDescription::OPTIONAL)==1 && i<(tl.length()-1) && tl[i+1].type==Token::openSquare){ // next Token needs to be [ i.e. optional arg, otherwise start verbatim directly
                                         // special treatment for \begin{abc}[...]
                                         cd.verbatimAfterOptionalArg=true;
-                                        cd.args--;
-                                        cd.argTypes.takeFirst();
+                                        cd.arguments.takeFirst();
                                         cd.optionalCommandName="\\begin{" + env + "}";
                                         cd.level=tk1.level;
                                         commandStack.push(cd);
@@ -519,12 +639,12 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                                         tk3.dlh = dlh;
                                         tk3.level = level - 1;
                                         tk3.type = Token::verbatim;
+                                        tk3.optionalCommandName=env; // store verbatim env name (fix #2386, \end{diffVerbatim} was falsely used to close verbatim)
                                         stack.push(tk3);
                                     }
                                 } else { // only care for further arguments if not in verbatim mode (see minted)
-                                    if ((cd.args > 1)||(cd.args==1 && cd.optionalArgs>0)) {
-                                        cd.args--;
-                                        cd.argTypes.takeFirst();
+                                    if ((cd.args() > 1)||(cd.args()==1 && cd.args(ArgumentDescription::OPTIONAL)>0)) {
+                                        cd.arguments.takeFirst();
                                         cd.optionalCommandName="\\begin{" + env + "}";
                                         cd.level=tk1.level;
                                         commandStack.push(cd);
@@ -554,10 +674,14 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                     tk.subtype=tk1.subtype;
                     level=tk1.level; // restore original level
                     lexed.append(tk);
+                    // clean up command stack (unrealized arguments)
+                    while (!commandStack.isEmpty() && commandStack.top().level > level) {
+                        commandStack.pop();
+                    }
                 }
                 if (!commandStack.isEmpty() && commandStack.top().level == level) {
-                    CommandDescription &cd = commandStack.top();
-                    if (cd.args <= 0 && cd.bracketArgs <= 0) {
+                    CommandDescription cd = commandStack.top();
+                    if (cd.arguments.isEmpty()) {
                         // all args handled, stop handling this command
                         commandStack.pop();
                         if(cd.verbatimAfterOptionalArg){ // delayed verbatim start to handle optional argument
@@ -566,6 +690,8 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                             tk3.dlh = dlh;
                             tk3.level = level - 1;
                             tk3.type = Token::verbatim;
+                            QString env=cd.optionalCommandName.mid(7,cd.optionalCommandName.length()-8); // dirty solution, does not use tokens as it should
+                            tk3.optionalCommandName=env;
                             stack.push(tk3);
                         }
                     }
@@ -600,6 +726,7 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                         Token tk0;
                         tk0.start = tk.start;
                         tk0.length = 0;
+                        tk0.dlh=dlh;
                         tk0.type = Token::keyVal_val;
                         tk0.subtype = Token::keyVal_val;
                         tk0.level = level;
@@ -614,13 +741,24 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                 //lastComma = -1;
                 lastEqual = level;
                 level++;
+                if(!commandStack.isEmpty() && lp->commandDefs.contains(commandStack.top().optionalCommandName + "/" + keyName)){
+                    // handle keyval values with normal commandStack mechanism (mandatory argument only!!)
+                    // allows argument classification
+                    CommandDescription cd = lp->commandDefs.value(commandStack.top().optionalCommandName + "/" + keyName);
+                    if(cd.arguments[0].tokenType!=Token::definition){
+                        // special treatment for length for now
+                        continue;
+                    }
+                    cd.level=level;
+                    commandStack.push(cd);
+                }
                 continue;
             }
             if (lastComma < 0 ) {
                 tk.level = level;
                 tk.type = Token::keyVal_key;
                 if(!commandStack.isEmpty()){
-                    CommandDescription &cd = commandStack.top();
+                    const CommandDescription &cd = commandStack.top();
                     tk.optionalCommandName=cd.optionalCommandName;
                 }
                 keyName = line.mid(tk.start, tk.length);
@@ -643,6 +781,15 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                             }
                         }
                     }
+                    // special treatment for numbers after word (join them to word, see https://github.com/texstudio-org/texstudio/issues/2306 )
+                    if (tk.type == Token::number) {
+                        if(lastComma==(lexed.length()-2)){
+                            if((lexed[lastComma+1].start+lexed[lastComma+1].length==tk.start)&&lexed[lastComma+1].type==Token::word){
+                                lexed[lastComma+1].length+=tk.length;
+                                continue;
+                            }
+                        }
+                    }
                     // special treatment for $ as mathstart
                     if (line.mid(tk.start, 1) == "$") {
                         tk.type = Token::command;
@@ -669,9 +816,11 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
                             }
                         }
                     }
-                    if (!commandStack.isEmpty() && lp.commandDefs.contains(commandStack.top().optionalCommandName + "/" + keyName)) {
-                        CommandDescription cd = lp.commandDefs.value(commandStack.top().optionalCommandName + "/" + keyName);
-                        tk.type = cd.argTypes.value(0, Token::keyVal_val); // only types can be set in key_val as they need to be recognized later
+                    //basically used for length values
+                    if (!commandStack.isEmpty() && lp->commandDefs.contains(commandStack.top().optionalCommandName + "/" + keyName)) {
+                        CommandDescription cd = lp->commandDefs.value(commandStack.top().optionalCommandName + "/" + keyName);
+                        auto ad=cd.arguments.value(0, ArgumentDescription{ArgumentDescription::MANDATORY,Token::keyVal_val});
+                        tk.type = ad.tokenType; // only types can be set in key_val as they need to be recognized later
                         if(!lexed.isEmpty() && lexed.last().type==tk.type && lexed.last().subtype==tk.subtype){
                             lexed.last().length=tk.start+tk.length-lexed.last().start;
                             continue;
@@ -734,16 +883,21 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
             if (!commandStack.isEmpty() && commandStack.top().level == level) {
                 //possible command argument without brackets
                 CommandDescription &cd = commandStack.top();
-                if (cd.args > 0) {
-                    cd.optionalArgs = 0; // no optional arguments after mandatory
-                    cd.args--;
-                    tk.subtype = cd.argTypes.takeFirst();
+                // skip over optional arguments, here assume optional arguments as [],<> and ()
+                while(!cd.arguments.isEmpty() && (cd.arguments.first().type >= ArgumentDescription::OPTIONAL) ) {
+                    cd.arguments.takeFirst();
                 }
-                if (cd.args <= 0) {
-                    // unknown arg, stop handling this command
+                if (cd.arguments.size() && cd.arguments.first().type==ArgumentDescription::MANDATORY) {
+                    auto ad=cd.arguments.takeFirst();
+                    tk.subtype = ad.tokenType;
+                    tk.level++; // needs tk level be increased
+                }else{
+                    cd.arguments.clear(); // unknown arg, stop handling this command
+                }
+                if (cd.arguments.isEmpty()) {
+                    // command has been handled completely
                     commandStack.pop();
                 }
-                tk.level++; // needs tk level be increased
             }
 
             if(tk.subtype==Token::keyVal_val && tk.type==Token::punctuation && line.mid(tk.start, tk.length)==",") continue; // exception for comma in keyVal braces
@@ -790,9 +944,7 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
     }
 
     dlh->setCookie(QDocumentLine::LEXER_COOKIE, QVariant::fromValue<TokenList>(lexed));
-    // run-away prevention
-    // reduce argLevel by 1, remove all elements with level <0
-    // TODO: needs to be applied on commandStack as well !!!
+
     for (int i = 0; i < stack.size(); i++) {
         if (stack[i].type == Token::verbatim)
             continue;
@@ -805,6 +957,9 @@ bool latexDetermineContexts2(QDocumentLineHandle *dlh, TokenStack &stack, Comman
     dlh->setCookie(QDocumentLine::LEXER_REMAINDER_COOKIE, QVariant::fromValue<TokenStack>(stack));
     dlh->setCookie(QDocumentLine::LEXER_COMMANDSTACK_COOKIE, QVariant::fromValue<CommandStack>(commandStack));
     dlh->setCookie(QDocumentLine::LEXER_COMMENTSTART_COOKIE, QVariant::fromValue<QPair<int,int> >({commentStart, Token::unknownComment}));
+    dlh->setFlag(QDocumentLine::lexedPass2InComplete,unknownCommandsPresent);
+    dlh->setFlag(QDocumentLine::lexedPass2Complete,!unknownCommandsPresent);
+    dlh->setFlag(QDocumentLine::argumentsParsed,false);
     dlh->unlock();
 
     bool remainderChanged = (stack != oldRemainder) || (commandStack != oldCommandStack) ;
@@ -1209,15 +1364,17 @@ TokenStack getContext(QDocumentLineHandle *dlh, int pos)
 	TokenStack ts;
 	for (int i = 0; i < tl.length(); i++) {
 		Token tk = tl.at(i);
-		if (tk.start > pos) {
+        if (tk.start > pos) {
 			break;
 		}
-		if (Token::tkOpen().contains(tk.type)) {
-			stack.push(tk);
-		}
+        if(tk.start == pos && tk.length>0){
+            // break if token is right of cursor, fix #3967
+            // also fix #4017
+            break;
+        }
         if (Token::tkClose().contains(tk.type) && !stack.isEmpty() ) {
             if (stack.top().type == Token::opposite(tk.type) && (tk.start<pos)) {
-				stack.pop();
+                stack.pop();
 			}
             continue;
         }
@@ -1344,7 +1501,7 @@ Token getCommandTokenFromToken(TokenList tl, Token tk)
  * \param column
  * \return 512 if token at column is 'width'
  */
-int getCompleterContext(QDocumentLineHandle *dlh, int column)
+EnumsTokenType::TokenType getCompleterContext(QDocumentLineHandle *dlh, int column)
 {
 	TokenStack ts = getContext(dlh, column);
 	Token tk;
@@ -1358,19 +1515,71 @@ int getCompleterContext(QDocumentLineHandle *dlh, int column)
 		}
 	}
 
-	Token::TokenType type = tk.type;
+    EnumsTokenType::TokenType type = tk.type;
 	if (tk.subtype != Token::none && tk.subtype != Token::keyVal_val)
 		type = tk.subtype;
 
-	int result = 0;
-	switch (type) {
-	case Token::width:
-		result = 512;
-		break;
-	default:
-		;
-	}
-	return result;
+    return type;
+}
+/*!
+ * \brief return argument as a token list
+ * \param tl initial token list, starting with the first token of an argument
+ * \param type which is searched
+ * \return argument as tokenlist
+ */
+TokenList getArgTL(const TokenList &tl, EnumsTokenType::TokenType type)
+{
+    for(int i=0;i<tl.size();++i) {
+        Token tk=tl.value(i);
+        if (tk.subtype==type) {
+            TokenList result;
+            if (Token::tkBraces().contains(tk.type)) {
+                // braces token and all within
+                int j=i+1;
+                for(;j<tl.size();++j){
+                    if(tl.value(j).start>(tk.start+tk.length)){
+                        break;
+                    }
+                }
+                result = tl.mid(i, j-i);
+            }
+            if (Token::tkOpen().contains(tk.type)) {
+                result = tl.mid(i) + findRestArgTL(tk.dlh, Token::opposite(tk.type), -1,ConfigManager::RUNAWAYLIMIT);
+            }
+            if (Token::tkClose().contains(tk.type)) {
+                result = tl.mid(0,i+1); // include closing token
+            }
+            return result;
+        }
+    }
+    return TokenList();
 }
 
+TokenList findRestArgTL(QDocumentLineHandle *dlh, Token::TokenType type, int hint, int count)
+{
+    // dlh is current line, next line will be checked here!!!
+    if (count <= 0)
+        return TokenList(); // limit search depth
+    QDocument *document = dlh->document();
+    int index = document->indexOf(dlh,hint);
+    if (index + 1 >= document->lines())
+        return TokenList(); // last line reached
+    dlh = document->line(index + 1).handle();
+    TokenList tl = dlh->getCookie(QDocumentLine::LEXER_COOKIE).value<TokenList>();
+    for (int i = 0; i < tl.length(); i++) {
+        Token tk = tl.at(i);
+        if (tk.type == type) {
+            // closing found
+            return tl.mid(0,i+1); // include closing token
+        }
+        if (Token::tkClose().contains(tk.type)) {
+            // wrong closing found/ syntax problem
+            //return value anyway
+            return tl.mid(0,i+1); // include closing token
+        }
+    }
+    return tl + findRestArgTL(dlh, type, index+1, count - 1);
+}
+
+
 }  // namespace Parsing
