@@ -25,8 +25,9 @@ struct PDFDocumentConfig {
 	double zoomStepFactor;
 
     int magnifierSize;
-    enum MagnifierShape {Rect,CircleWithShadow,Circle};
+    enum MagnifierShape {Rectangle,Circle};
     MagnifierShape magnifierShape;
+	bool magnifierShadow;
 	bool magnifierBorder;
 
 	int laserPointerSize;
@@ -35,15 +36,18 @@ struct PDFDocumentConfig {
 	QString syncFileMask;
 	QString paperColor; // hex rgb
 	QString highlightColor; // hex rgba, e.g. #FFFF003F
+    QString selectColor; // hex rgba, e.g. #FFFF003F
 	int highlightDuration;
 
 //live options
 	int editTool;
 	bool continuous, singlepagestep;
+	bool continuousEmbedded, singlepagestepEmbedded;
 	bool followFromCursor, followFromScroll, syncViews;
 	bool invertColors;
 	bool grayscale;
-	int gridx, gridy;
+	int gridx, gridy, pageOffset;
+	int gridxEmbedded, gridyEmbedded, pageOffsetEmbedded;
 };
 
 #endif // PDFDOCUMENT_CONFIG_H
